Somewhat better solution.

git-svn-id: https://llvm.org/svn/llvm-project/llvm/trunk@48170 91177308-0d34-0410-b5e6-96231b3b80d8
diff --git a/lib/CodeGen/SelectionDAG/DAGCombiner.cpp b/lib/CodeGen/SelectionDAG/DAGCombiner.cpp
index 5cda144..4592971 100644
--- a/lib/CodeGen/SelectionDAG/DAGCombiner.cpp
+++ b/lib/CodeGen/SelectionDAG/DAGCombiner.cpp
@@ -1103,9 +1103,10 @@
   
   // fold (sub x, x) -> 0
   if (N0 == N1) {
-    if (ISD::isBuildVectorAllZeros(N0.Val))
-      // Zero vectors might be normalized to a particular vector type to ensure
-      // they are CSE'd. Return it as it is.
+    if (!AfterLegalize || !MVT::isVector(VT))
+      // For example, zero vectors might be normalized to a particular vector
+      // type to ensure they are CSE'd. Avoid issuing zero vector nodes of
+      // *unexpected* type after legalization.
       return N0;
     return DAG.getConstant(0, N->getValueType(0));
   }